% In this script the 2008 article of Badel is closely followed
clearvars -except delete
close all

% Collecting all the required directories
directories = dir('/Users/stefanobordabossana/Documents/Experiments/analysed cells with tf/*');
directories(1:3) = [];
directories(end) = [];

for i = 1:length(directories) % Loop over different cells
    
    if delete(i)
        
        continue
        
    else
        
        cd([directories(i).folder '/' directories(i).name]);
        
        % Load the current recording
        files = listH5Files([pwd '/tf/']);
        kernels = dir([pwd '/tf/*_kernel.dat']);
        Ke = load(kernels(1).name);
        [t,Vr,Iinj,stim] = loadH5Traces(files);
        [Vc] = myAEC(Ke, Iinj, Vr, files, t, stim, 1);
        
        load([pwd '/vi/01/VI_curve.mat']);
        load([pwd '/tau/01/tau.mat']);
        
        
        taucell = struct2cell(tau);
        taumatr = cell2mat(taucell([1 2]));
        
        if max(taumatr) > 100
            timeconstant = min(taumatr);            
        else          
            timeconstant = max(taumatr);          
        end
        capacitance = timeconstant / VI.resistanceSteadyState;
        
        
        Iinj_step = Iinj(1,:);
        Iinj_step(Iinj_step ~= 0) = 1;
        % Extracting star/stop points of stimulations
        stimulation_StartPoints = strfind(Iinj_step, [0 1]);
        stimulation_EndPoints = strfind(Iinj_step, [1 0]);
        
        % Collecting the noisy input current, corresponding voltage trace
        % and extract the AP from the voltage trace
        Inoise = Iinj(1 ,stimulation_StartPoints(3)+1:stimulation_EndPoints(3));
        Vnoise = Vc(1 ,stimulation_StartPoints(3)+1:stimulation_EndPoints(3));
        [~, AP_locs] = findpeaks(Vnoise, 'MinPeakHeight', -20, 'MinPeakWidth', 20);
        
        % Deleting 200 ms after each AP
        timewindow = 200;
        rows = 0:(timewindow/t(2)/1000);
        windRange = repmat(rows, size(AP_locs,2), 1);
        windRange = windRange + AP_locs';
        delRange = unique(windRange);
        
        dvdt = diff(Vnoise) ./ (t(2) * 1000);
        delRange(delRange > size(Inoise, 2)) = [];
        Vnoise(delRange(1:end-1)) = [];
        Inoise(delRange(1:end-1)) = [];
        dvdt(delRange(1:end-1)) = [];
        
        % Sorting the voltage to create histogram
        [sortedVnoise, indexes] = sort(Vnoise(1:end-1));
        Inoise = Inoise(indexes);
        dvdt = dvdt(indexes);
        
        % Loading the threshold as upper limit for the calculations
        load([pwd '/tf/AP_noise.mat']);
        upLimit = AP_noise.threshold - 0.4 * AP_noise.threshold;
        
        sortedVnoise(sortedVnoise > upLimit) = [];
        Inoise = Inoise(1:size(sortedVnoise, 2));
        dvdt = dvdt(1:size(sortedVnoise, 2));

        % Calculating the capacitance of the neuron
        idxLowSixty = find(sortedVnoise > -60, 1, 'first');
        Ce = 0.01:0.01:10;
        C = zeros(size(Ce));
        for k = 1:size(Ce,2)
            C(k) = var(Inoise(1:idxLowSixty) - Ce(k) .* dvdt(1:idxLowSixty));
        end
        C = Ce(C == min(C));
        if C == min(Ce)
            C = capacitance;        
        end
        
        Iion = Inoise(1:end) - (C) .* dvdt; % The ionic currnet

        % Binning the dynamic current
        bins = 100;
        MSVN = zeros(bins, 1);
        MSII = zeros(bins, 1);
        for k = 1:bins
            binsize = (max(sortedVnoise) - min(sortedVnoise)) / bins;
            startbin = binsize * (k-1) + min(sortedVnoise);
            endbin = min(sortedVnoise) + binsize * k;
            
            MSVN(k) = mean(sortedVnoise((startbin < sortedVnoise) & (sortedVnoise < endbin)));
            MSII(k) = mean(Iion((startbin < sortedVnoise) & (sortedVnoise < endbin)));
        end
        
        Idyn = MSII;
        FV = -Idyn / (C);
        
        % Custom written function to delete eventual outliers from the
        % fitting routine
        [MSVNdel, FVdel, allIndsDel] = dataPreparation(MSVN', FV');
        MSVNdel(isnan(MSVNdel)) = [];
        FVdel(isnan(FVdel)) = [];
        
        % Fitting the EIF to the dynamic current
        ft = fittype('(1/a) * (b - x + c * exp((x-d)/c))', 'independent', 'x', 'dependent', 'y');
        opts = fitoptions('Method', 'NonlinearLeastSquares');
        opts.Display = 'Off';
        opts.StartPoint = [5, -70, 5, -50];
        [fitresult, gof] = fit(MSVNdel', FVdel', ft, opts);
        coeff = coeffvalues(fitresult);
        
        fitX = min(MSVNdel):0.01:max(MSVNdel);
        fitY = (1/coeff(1)) * (coeff(2) - fitX + coeff(3) * exp((fitX-coeff(4))/coeff(3)));
        
        % Plotting the data
        figure(2)
        set(gcf, 'color', 'white');
        scatter(MSVNdel, FVdel, 60, 'markeredgecolor', 'k', 'linewidth', 1.5);
        hold on
        plot(fitX, fitY, 'r', 'linewidth', 2);
        ax2 = gca;
        ax2.FontSize = 16;
        ax2.FontName = 'Arial';
        xlabel('V [mV]', 'FontSize', 20, 'FontName', 'Arial');
        ylabel('F(V) [mV/ms]', 'FontSize', 20, 'FontName', 'Arial');
        
        % Saving the data
        dynFit.capacitance = C;
        dynFit.tau = coeff(1);
        dynFit.EL = coeff(2);
        dynFit.deltaT = coeff(3);
        dynFit.VT = coeff(4);
        dynFit.gof = gof;
        
        pathtosave = [pwd '/tf/'];
        saveas(figure(1), fullfile ([pathtosave '/'], 'dynamicfit'), 'fig');
        save([pathtosave '/dynFit.mat'], 'dynFit');
        
        display(dynFit)
        
        close all
        clearvars -except directories delete
        
        
    end
    
end